/*
	Description: Creates Figure 1 (graph showing changes in manufacturing and non-manufacturing employment)

	Author: Brendan Price
	Updated: July 30, 2014
*/

version 12.1
set more off
clear all
capture restore, not
set scheme s1color

local outdir "../../out"

* Load the estimation sample
use "../../dta/sample/main.dta", clear

keep sic87dd manuf cbp_emp19* cbp_emp20*

forvalues i = 1991/2011 {
	* Compute total employment in manufacturing, in non-manufacturing, and in aggregate
	egen manuf_emp`i' = total(cbp_emp`i' * (manuf == 1))
	egen nonmanuf_emp`i' = total(cbp_emp`i' * (manuf == 0))
	egen tot_emp`i' = total(cbp_emp`i')
}

keep manuf_emp* nonmanuf_emp* tot_emp*
duplicates drop
assert _N == 1

* Retain baseline values for normalization
gen baseline_manuf_emp = manuf_emp1991
gen baseline_nonmanuf_emp = nonmanuf_emp1991
gen baseline_tot_emp = tot_emp1991

gen i = 1
reshape long manuf_emp nonmanuf_emp tot_emp, i(i) j(year)
drop i

* Export employment counts to Excel
label var year "Year"
label var manuf_emp "Manuf Emp"
label var nonmanuf_emp "Non-Manuf Emp"
label var tot_emp "Total Emp"

export excel year manuf_emp nonmanuf_emp tot_emp using "`outdir'/AADHP-JOLE-Tables.xlsx", sheetreplace sheet(Fig1_Raw) firstrow(varlabels)

* Normalize baseline employment values to unity
foreach v in "manuf_emp" "nonmanuf_emp" "tot_emp"  {
	replace `v' = `v'/baseline_`v'
	drop baseline_`v'
}

* Employment plot
#delimit ;
twoway (connected manuf_emp year, clcolor(navy) clpattern(solid) mcolor(navy) msize(medium) msymbol(circle))
	(connected nonmanuf_emp year, clcolor(maroon) clpattern(longdash) mcolor(maroon) msize(medium) msymbol(diamond))
	(connected tot_emp year, clcolor(dkorange) clpattern(shortdash) mcolor(dkorange) msize(medium) msymbol(square)),
	title("")
	xtitle("")
	ytitle("Employment: 1991 = 1", size(medsmall))
	xlabel(1991(4)2011, labsize(medsmall))
	xmtick(1991(2)2011)
	yscale(range(.5 1.5))
	ylabel(.5(.1)1.5, angle(0) labsize(medsmall) grid)
	ymtick(.5(.05)1.5)
	legend(size(medsmall) rows(3)
		label(1 "U.S. Manufacturing Employment")
		label(2 "U.S. Non-Manufacturing Employment")
		label(3 "U.S. Total Employment"));
#delimit cr

graph export "`outdir'/figs/Fig1.eps", replace
!epstopdf --outfile="`outdir'/figs/Fig1.pdf" "`outdir'/figs/Fig1.eps"


