/*
	Description: Construct measures of upstream and downstream import exposure

	Author: Brendan Price
	This version: February 1, 2015
	
	Note on nomenclature: sic87dd_down denotes industries purchasing the good; sic87dd_up denotes industries producing the good.
*/

version 12.1
set more off
set matsize 1000
clear all

local iodir "../../../dta/io"
local tradedir "../../../dta/trade"

/* Compute first-order upstream trade exposure (propagating upstream from an industry's customers) and downstream trade exposure (propagating downstream from its suppliers) */

foreach d in "up" "down" {
	* Load the matrix of input output data
	use "`iodir'/clean/shares/gross_shares.dta", clear

	if "`d'" == "up" {
		* Compute and retain the fraction of sales that go to (1) manuf, (2) non-manuf, and (3) residual category
		bysort sic87dd_up: egen manuf_sales = total(produce_share * (sic87dd_down >= 2000 & sic87dd_down <= 3999))
		bysort sic87dd_up: egen nonmanuf_sales = total(produce_share * (sic87dd_down < 2000 | (sic87dd_down > 3999 & sic87dd_down < .)))
		bysort sic87dd_up: egen resid_sales = total(produce_share * (sic87dd_down == .))

		label var manuf_sales "Share of sales made to mfg industries"
		label var nonmanuf_sales "Share of sales made to non-mfg industries"
		label var resid_sales "Share of sales made to residual categories"
		local extra_vars "manuf_sales nonmanuf_sales resid_sales"

		* Sales shares should sum to unity
		assert abs(1 - (manuf_sales + nonmanuf_sales + resid_sales)) < .001

		* Integrating out downstream firms
		rename sic87dd_down sic87dd
		local w "produce_share"

		* Variable that will remain after "integration"
		local indvar "sic87dd_up"
	}
	else if "`d'" == "down" {
		* Compute and retain the fraction of purchases that are from (1) manuf, (2) non-manuf, and (3) residual category
		bysort sic87dd_down: egen manuf_purchases = total(purchase_share * (sic87dd_up >= 2000 & sic87dd_up <= 3999))
		bysort sic87dd_down: egen nonmanuf_purchases = total(purchase_share * (sic87dd_up < 2000 | (sic87dd_up > 3999 & sic87dd_up < .)))
		bysort sic87dd_down: egen resid_purchases = total(purchase_share * (sic87dd_up == .))

		label var manuf_purchases "Purchases from mfg industries divided by total sales"
		label var nonmanuf_purchases "Purchases from non-mfg industries divided by total sales"
		label var resid_purchases "Purchases from residual categories divided by total sales"
		local extra_vars "manuf_purchases nonmanuf_purchases resid_purchases"

		* Note that "purchase shares" don't sum to unity because they are normalized by total SALES, not total purchases.
		* However, it's important that these shares sum to less than unity across manufacturing.
		assert manuf_purchases <= 1

		* Integrating out upstream firms
		rename sic87dd_up sic87dd
		local w "purchase_share"

		* Variable that will remain after "integration"
		local indvar "sic87dd_down"
	}

	* Merge in trade variables (master observations should match manufacturing sector only)
	merge m:1 sic87dd using "`tradedir'/trade_variables.dta", keepusing(d_import_usch_* d_import_otch_* d_export_usch_* d_export_otch_* d_net_import_usch_* d_net_import_otch_*) assert(1 3)
	assert _merge == 3 if sic87dd >= 2000 & sic87dd <= 3999
	drop _merge

	* Trade variables should be missing for all non-manuf industries, non-missing for all manuf industries.
	* Treat non-manufacturing industries as experiencing null trade exposure in all periods.
	foreach var of varlist d_import_* d_export_* d_net_import_* {
		assert `var' != . if sic87dd >= 2000 & sic87dd <= 3999
		assert `var' == . if sic87dd < 2000 | sic87dd > 3999
		replace `var' = 0 if `var' == .
	}

	foreach span in 1991_2011 1991_1999 1999_2007 1999_2011 2007_2011 1991_2007 1991_2009 1999_2009 {
		* Construct measures of upstream/downstream import exposure
		bysort `indvar': egen d_`d'_usch_`span' = total(`w' * d_import_usch_`span')
		label var d_`d'_usch_`span' "First-order `d'stream import exposure defined using gross flows"

		* Instrument for upstream/downstream import exposure
		bysort `indvar': egen d_`d'_otch_`span' = total(`w' * d_import_otch_`span')
		label var d_`d'_otch_`span' "First-order `d'stream instrument defined using gross flows"
	}

	* Drop the "residual" sector (retained up until now to handle manufacturing industries that only ship to this sector)
	drop if `indvar' == .
	
	* Save a dataset of indirect import exposure measures
	keep `indvar' d_`d'_* `extra_vars'
	rename `indvar' sic87dd
	duplicates drop

	tempfile first_order_exposure_`d'
	save "`first_order_exposure_`d''.dta", replace
}

use "`first_order_exposure_up'.dta", clear
merge 1:1 sic87dd using "`first_order_exposure_down'.dta", assert(3) nogenerate
compress
save "`iodir'/clean/exposure/first_order_exposure.dta", replace

/* Now compute "full" measures of upstream/downstream import exposure incorporating higher-order effects */

foreach d in "up" "down" {
	* Load the matrix of input output data
	use "`iodir'/clean/shares/leontief_shares.dta", clear

	if "`d'" == "up" {
		* Integrating out downstream firms
		rename sic87dd_down sic87dd
		local w "produce_share"

		* Variable that will remain after "integration"
		local indvar "sic87dd_up"
	}
	else if "`d'" == "down" {
		* Integrating out upstream firms
		rename sic87dd_up sic87dd
		local w "purchase_share"
		
		* Variable that will remain after "integration"
		local indvar "sic87dd_down"
	}
	
	* Merge in trade variables (master observations should match manufacturing sector only)
	merge m:1 sic87dd using "`tradedir'/trade_variables.dta", keepusing(d_import_usch_* d_import_otch_* d_export_usch_* d_export_otch_* d_net_import_usch_* d_net_import_otch_*) assert(1 3)
	assert _merge == 3 if sic87dd >= 2000 & sic87dd <= 3999
	drop _merge

	* Trade variables should be missing for all non-manuf industries, non-missing for all manuf industries.
	* Treat non-manufacturing industries as experiencing null trade exposure in all periods.
	foreach var of varlist d_import_* {
		assert `var' != . if sic87dd >= 2000 & sic87dd <= 3999
		assert `var' == . if sic87dd < 2000 | sic87dd > 3999
		replace `var' = 0 if `var' == .
	}
	
	foreach span in 1991_2011 1991_1999 1999_2007 1999_2011 2007_2011 1991_2007 1991_2009 1999_2009 {
		* Construct measures of upstream/downstream import exposure
		bysort `indvar': egen d_`d'_LT_usch_`span' = total(`w' * d_import_usch_`span')
		label var d_`d'_LT_usch_`span' "Full `d'stream import exposure defined using gross flows"

		* Instrument for upstream/downstream import exposure
		bysort `indvar': egen d_`d'_LT_otch_`span' = total(`w' * d_import_otch_`span')
		label var d_`d'_LT_otch_`span' "Full `d'stream instrument defined using gross flows"
	}
	
	keep `indvar' d_`d'_LT_*
	rename `indvar' sic87dd
	duplicates drop

	* Save a dataset of indirect import exposure measures
	tempfile leontief_exposure_`d'
	save "`leontief_exposure_`d''.dta", replace
}

use "`leontief_exposure_up'.dta", clear
merge 1:1 sic87dd using "`leontief_exposure_down'.dta", assert(3) nogenerate
compress
save "`iodir'/clean/exposure/leontief_exposure.dta", replace



