/*
	Description: Creates Table 3 (2SLS import effects on employment accounting for numerous industry-level controls)

	Author: Brendan Price
	Updated: July 30, 2014
*/

version 12.1
set more off
clear all

local outdir "../../out"

* Partial R-squared used to "discount" implied employment changes
local pfactor = .56

* Predict implied employment change
capture program drop compute_changes
program define compute_changes
	egen tot_pred_d_emp_t1 = total(pred_d_emp * t1)
	egen tot_pred_d_emp_t2 = total(pred_d_emp * t2)
	egen tot_pred_d_emp = total(pred_d_emp)

	foreach d in tot_pred_d_emp_t1 tot_pred_d_emp_t2 tot_pred_d_emp {
		quietly sum `d'
		estadd scalar `d' = r(mean)
	}
		
	drop tot_pred_d_emp_t1 tot_pred_d_emp_t2 tot_pred_d_emp
	replace pred_d_emp = 0
end

* Load the estimation sample
use "../../dta/sample/main.dta", clear

* Restrict to manufacturing
keep if manuf == 1

* Specify controls
local sector_controls "sector2 sector3 sector4 sector5 sector6 sector7 sector8 sector9 sector10"
local prod_controls "ind_prodwemp_1991 ind_lnavgw_1991 ind_capvadd_1991 ind_ci_1990 ind_htsh1_1990"
local pretrend_controls "d_ind_shemp_7691 d_ind_lnavgw_7691"

* Restrict to variables we need
#delimit ;
keep sic87dd sic3 cbp_emp1991 cbp_emp1999 cbp_emp2011 `sector_controls' `prod_controls' `pretrend_controls'
	dl_cbp_emp_1991_1999 dl_cbp_emp_1999_2011
	d_import_usch_1991_1999 d_import_usch_1999_2011
	d_import_otch_1991_1999 d_import_otch_1999_2011;
#delimit cr

* Stacked first differences
reshape long dl_cbp_emp_ d_import_usch_ d_import_otch_, i(sic87dd) j(period) string
rename dl_cbp_emp_ dl_cbp_emp
rename d_import_usch_ d_import_usch
rename d_import_otch_ d_import_otch

* Time dummies
gen t1 = (period == "1991_1999")
gen t2 = (period == "1999_2011")

* Label regressors
label var t1 "1{1991-1999}"
label var t2 "1{1999-2011}"

* End-of-period employment for use in employment calculations
gen L1 = cbp_emp1999 if period == "1991_1999"
replace L1 = cbp_emp2011 if period == "1999_2011"

* Specify weighting and options
local aw "[aw = cbp_emp1991]"
local options "noconstant vce(cluster sic3)"

* Demean all control variables so that changes in the time effects are interpretable
foreach var of varlist `sector_controls' `prod_controls' `pretrend_controls' {
	forvalues i = 1/2 {
		quietly sum `var' if t`i' == 1 `aw'
		replace `var' = `var' - r(mean) if t`i' == 1
	}
}

* Specify outcome/regressor/instrument
local yvar "dl_cbp_emp"
local xvar "d_import_usch"
local ivvar "d_import_otch"

/* Regression specs */
estimates clear

* Spec w/out controls
eststo m1: quietly ivregress 2sls `yvar' t1 t2 (`xvar' = `ivvar') `aw', `options'
estadd local industry_fes "No"

* Add sector controls
eststo m2: quietly ivregress 2sls `yvar' t1 t2 `sector_controls' (`xvar' = `ivvar') `aw', `options'
estadd local industry_fes "No"

* Add production controls
eststo m3: quietly ivregress 2sls `yvar' t1 t2 `prod_controls' (`xvar' = `ivvar') `aw', `options'
estadd local industry_fes "No"

* Add pretrend controls
eststo m4: quietly ivregress 2sls `yvar' t1 t2 `pretrend_controls' (`xvar' = `ivvar') `aw', `options'
estadd local industry_fes "No"

* Sector + production controls
eststo m5: quietly ivregress 2sls `yvar' t1 t2 `sector_controls' `prod_controls' (`xvar' = `ivvar') `aw', `options'
estadd local industry_fes "No"

* Sector + pretrend controls
eststo m6: quietly ivregress 2sls `yvar' t1 t2 `sector_controls' `pretrend_controls' (`xvar' = `ivvar') `aw', `options'
estadd local industry_fes "No"

* All controls
eststo m7: quietly ivregress 2sls `yvar' t1 t2 `sector_controls' `prod_controls' `pretrend_controls' (`xvar' = `ivvar') `aw', `options'
estadd local industry_fes "No"

* Industry fixed effects
xtset sic87dd
eststo m8: quietly xtivreg2 `yvar' t2 (`xvar' = `ivvar') `aw', fe cluster(sic3)
estadd local industry_fes "Yes"

/* Compute implied employment effects */

gen pred_d_emp = 0

* Factors needed to rescale coefficients (to undo multiplication by 100 and also annualization)
gen lambda = 100 / (1999 - 1991) if period == "1991_1999"
replace lambda = 100 / (2011 - 1999) if period == "1999_2011"
assert lambda != .

* Compute implied employment effects for the specification with 1-digit controls only
estimates restore m2
replace pred_d_emp = L1 * (1 - exp(-1 * (1/lambda) * (_b[`xvar'] * `pfactor' * `xvar')))
compute_changes

/* Print results */

tempfile specs

#delimit ;
esttab * using "`specs'.csv",
	nodepvars
	collabels(, none)
	mlabels(, none)
	b(%9.2f) se(%9.2f)
	star(* .10 ** .05 *** .01)
	title("2SLS Estimates Including Industry-Level Controls")
	order(`xvar' t1 t2)
	nonotes
	indicate("Sector Controls = `sector_controls'" "Production Controls = `prod_controls'" "Pretrend Controls = `pretrend_controls'")
	label
	stats(industry_fes N tot_pred_d_emp_t1 tot_pred_d_emp_t2 tot_pred_d_emp, labels("Industry Fixed Effects" "N" "Pred Emp Change, 1991-1999" "Pred Emp Change, 1999-2011" "Pred Emp Change, 1991-2011") fmt(0 0 0 0 0))	
	replace;
#delimit cr

* Switch the output to Excel format
insheet using "`specs'.csv", comma clear
export excel "`outdir'/AADHP-JOLE-Tables.xlsx", sheetreplace sheet(Tab3_Raw)



